/***********************************************************************************
Matriz.java - classe para manusear matrizes.

   WRITTEN BY: Dr Michael Thomas Flanagan

   DATE:	 June 2002
   UPDATE:  21 April 2004, 19 January 2005, 1 May 2005

   DOCUMENTATION:
   See Michael Thomas Flanagan's Java library on-line web page:
   Matrix.html

   Copyright (c) April 2004  Michael Thomas Flanagan

   PERMISSION TO COPY:
   Permission to use, copy and modify this software and its documentation for
   NON-COMMERCIAL purposes is granted, without fee, provided that an acknowledgement
   to the author, Michael Thomas Flanagan at www.ee.ucl.ac.uk/~mflanaga, appears in 
   all copies.

   Dr Michael Thomas Flanagan makes no representations about the suitability
   or fitness of the software for any or for a particular purpose.
   Michael Thomas Flanagan shall not be liable for any damages suffered
   as a result of using, modifying or distributing this software or its derivatives.


**********************************************************************************

Parte integrante do Comunicado Tcnico 072/06.

Descrio:

  Classe Matriz

   Define uma matriz e inclui os mtodos necessrios para a manipulao padro de
   matrizes, e.g. multiplicao, e procedimentos relacionados, e.g. soluo de
   equaes lineares simultneas.

Parmetros de entrada:
  
  No se aplica.

Sada:

  No se aplica.

Informaes do CVS:
       $Source$:
       $Revision$:
       $Date$:

***********************************************************************************/

// pacotes genricos
import java.text.DecimalFormat;
import java.util.Arrays;

public class Matriz {

  // DATA VARIABLES
	private int nrow = 0;               	// number of rows
	private int ncol = 0;               	// number of columns
	private double matriz[][] = null; 	    // 2-D  Matriz
	private int index[] = null;            	// row permutation index
	private double dswap = 1.0D;           	// row swap index
	private boolean matrizCheck = true;     // check on matrix status
	                                	    // true - no problems encountered
	                                	    // false - attempted a LU decomposition on a singular matrix

	private static final double TINY = 1.0e-30;

	// CONSTRUCTORS
	// Construct a nrow x ncol matrix of complex variables all equal to zero
  public Matriz(int nrow, int ncol){
   	this.nrow = nrow;
		this.ncol = ncol;
		this.matriz = new double[nrow][ncol];
		this.index = new int[nrow];
		for (int i = 0; i < nrow; i++)
			this.index[i] = i;
  }

	// Construct a nrow x ncol matrix of complex variables all equal to 
	// the complex number const
  public Matriz(int nrow, int ncol, double constant){
   	this.nrow = nrow;
		this.ncol = ncol;
		this.matriz = new double[nrow][ncol];
		for (int i = 0; i < nrow; i++){
			for (int j = 0; j < nrow; j++)
				this.matriz[i][j] = constant;
		}
		this.index = new int[nrow];
		for (int i = 0; i < nrow; i++)
			this.index[i] = i;
 	}

	// Construct matrix with a reference to an existing nrow x ncol 2-D 
	// array of complex variables
  public Matriz(double[][] twoD){
		this.nrow = twoD.length;
		this.ncol = twoD[0].length;
		for (int i = 0; i < nrow; i++){
		    if (twoD[i].length != ncol)
		    	throw new IllegalArgumentException("All rows must have the same length");
		}
		this.matriz = twoD;
		this.index = new int[nrow];
		for (int i = 0; i < nrow; i++)
			this.index[i] = i;
	}

	// Construct matrix with a reference to the 2D matrix and 
	// permutation index of an existing ComplexMatriz bb.
  public Matriz(Matriz bb){
		this.nrow = bb.nrow;
		this.ncol = bb.ncol;
		this.matriz = bb.matriz;
		this.index = bb.index;
		this.dswap = bb.dswap;
 	}

    
  // METHODS
  // SET VALUES
  // Set the matrix with a copy of an existing nrow x ncol 2-D matrix of 
  // complex variables
  public void setTwoDarray(double[][] aarray){
   	String msgA = "row length of this ComplexMatriz differs from that ";
    String msgA1 = "of the 2D array argument";
		String msgB = "column length of this ComplexMatriz differs from that ";
    String msgB1 = "of the 2D array argument";
		String msgC = "All rows must have the same length";
		if (this.nrow != aarray.length)
			throw new IllegalArgumentException(msgA + msgA1);
		if (this.ncol != aarray[0].length)
			throw new IllegalArgumentException(msgB + msgB1);
		for (int i = 0; i < nrow; i++){
			if (aarray[i].length != ncol)
				throw new IllegalArgumentException(msgC);
			for (int j = 0; j < ncol; j++){
				this.matriz[i][j] = aarray[i][j];
			}
		}
	}

  // Set an individual array element
  // i = row index
  // j = column index
  // aa = value of the element
  public void setElement(int i, int j, double aa){
   	this.matriz[i][j] = aa;
  }

  // Set a sub-matrix starting with column index i, row index j
  // and ending with column index k, row index l
  public void setSubMatriz(int i, int j, int k, int l, double[][] smat){
   	if (i > k)
			throw new IllegalArgumentException("row indices inverted");
    if (j > l)
    	throw new IllegalArgumentException("column indices inverted");
    int n = k-i+1, m = j-l+1;
    for (int p = 0; p < n; p++){
    	for (int q = 0; q < m; p++){
    		this.matriz[i+p][j+q] = smat[i][j];
    	}
    }
  }

  // Set a sub-matrix
  // row = array of row indices
  // col = array of column indices
  public void setSubMatriz(int[] row, int[] col, double[][] smat){
   	int n = row.length;
   	int m = col.length;
   	for (int p = 0; p < n; p++){
   		for (int q = 0; q < m; p++){
   			this.matriz[row[p]][col[q]] = smat[p][q];
   		}
   	}
  }

  // Get the value of matrixCheck
  public boolean getMatrizCheck(){
   	return this.matrizCheck;
  }

  // SPECIAL MATRICES
  // Construct an identity matrix
  public static Matriz identityMatriz(int nrow){
   	Matriz u = new Matriz(nrow, nrow);
   	for (int i = 0; i < nrow; i++){
   		u.matriz[i][i] = 1.0;
   	}
   	return u;
  }

  // Construct a complex scalar matrix
  public static Matriz scalarMatriz(int nrow, double diagconst){
   	Matriz u = new Matriz(nrow, nrow);
   	double[][] uarray = u.getArrayReference();
   	for (int i = 0; i < nrow; i++){
   		for (int j = i; j < nrow; j++){
   			if (i == j){
   				uarray[i][j] =  diagconst;
   			}
   		}
   	}
   	return u;
  }

  // Construct a diagonal matrix
  public static Matriz diagonalMatriz(int nrow, double[] diag){
   	String msgA = "matriz dimension differs from diagonal array length";
   	if (diag.length != nrow)
   		throw new IllegalArgumentException(msgA);
   	Matriz u = new Matriz(nrow, nrow);
   	double[][] uarray = u.getArrayReference();
   	for (int i = 0; i < nrow; i++){
   		uarray[i][i] = diag[i];
   	}
   	return u;
  }

  // GET VALUES
  // Return the number of rows
  public int getNrow(){
   	return this.nrow;
  }

  // Return the number of columns
  public int getNcol(){
   	return this.ncol;
  }

  // Return a reference to the internal 2-D array
  public double[][] getArrayReference(){
   	return this.matriz;
  }

  // Return a reference to the internal 2-D array
  // included for backward compatibility with incorrect earlier documentation
  public double[][] getArrayPointer(){
   	return this.matriz;
  }

  // Return a copy of the internal 2-D array
  public double[][] getArrayCopy(){
  	double[][] c = new double[this.nrow][this.ncol];
		for (int i = 0; i < nrow; i++){
			for (int j = 0; j < ncol; j++){
				c[i][j] = this.matriz[i][j];
			}
		}
		return c;
  }

  // Return  a single element of the internal 2-D array
  public double getElement(int i, int j){
   	return this.matriz[i][j];
  }

  // Return a single element of the internal 2-D array
  // included for backward compatibility with incorrect earlier documentation
  public double getElementCopy(int i, int j){
  	return this.matriz[i][j];
  }

  // Return a single element of the internal 2-D array
  // included for backward compatibility with incorrect earlier documentation
  public double getElementPointer(int i, int j){
   	return this.matriz[i][j];
  }

  // Return a sub-matrix starting with column index i, row index j
  // and ending with column index k, row index l
  public Matriz getSubMatriz(int i, int j, int k, int l){
   	if (i > k)
  		throw new IllegalArgumentException("row indices inverted");
  	if (j > l)
   		throw new IllegalArgumentException("column indices inverted");
   	int n = k-i+1, m = j-l+1;
   	Matriz smat = new Matriz(n, m);
   	double[][] sarray = this.getArrayReference();
   	for (int p = 0; p < n; p++){
   		for (int q = 0; q < m; p++){
   			sarray[p][q] =  this.matriz[i+p][j+q];
   		}
   	}
   	return smat;
  }

  // Return a sub-matrix
  // row = array of row indices
  // col = array of column indices
  public Matriz getSubMatriz(int[] row, int[] col){
   	int n = row.length;
   	int m = col.length;
   	Matriz smat = new Matriz(n, m);
  	double[][] sarray = this.getArrayReference();
   	for (int i = 0; i < n; i++){
   		for (int j = 0; j < m; j++){
   			sarray[i][j] =  this.matriz[row[i]][col[j]];
   		}
   	}
   	return smat;
  }

  // Return a reference to the permutation index array
  public int[]  getIndexReference(){
   	return this.index;
  }

  // Return a reference to the permutation index array
  // included for backward compatibility with incorrect earlier documentation
  public int[]  getIndexPointer(){
   	return this.index;
  }

  // Return a copy of the permutation index array
  public int[]  getIndexCopy(){
   	int[] indcopy = new int[this.nrow];
   	for (int i = 0; i < this.nrow; i++){
   		indcopy[i] = this.index[i];
   	}
   	return indcopy;
  }

  // Return the row swap index
  public double getSwap(){
   	return this.dswap;
  }

  // COPY
  // Copy a Matriz [static method]
  public static Matriz copy(Matriz a){
   	if (a == null){
   		return null;
   	} else {
   		int nr = a.getNrow();
   		int nc = a.getNcol();
   		double[][] aarray = a.getArrayReference();
   		Matriz b = new Matriz(nr,nc);
   		b.nrow = nr;
   		b.ncol = nc;
   		double[][] barray = b.getArrayReference();
   		for (int i = 0; i < nr; i++){
   			for (int j = 0; j < nc; j++){
   				barray[i][j] = aarray[i][j];
   			}
   		}
   		for (int i = 0; i < nr; i++)
   			b.index[i] = a.index[i];
			
   		return b;
   	}
  }

  // Copy a Matriz [instance method]
  public Matriz copy(){
   	if (this == null){
   		return null;
   	} else {
   		int nr = this.nrow;
   		int nc = this.ncol;
   		Matriz b = new Matriz(nr,nc);
   		double[][] barray = b.getArrayReference();
   		b.nrow = nr;
  		b.ncol = nc;
   		for (int i = 0; i < nr; i++){
   			for (int j = 0; j < nc; j++){
   				barray[i][j] = this.matriz[i][j];
   			}
   		}
   		for (int i = 0; i < nr; i++)
    		b.index[i] = this.index[i];
			
   		return b;
   	}
  }

  // Clone a Matriz
  public Object clone(){
   	if (this == null){
   		return null;
   	} else {
   		int nr = this.nrow;
   		int nc = this.ncol;
   		Matriz b = new Matriz(nr,nc);
   		double[][] barray = b.getArrayReference();
   		b.nrow = nr;
   		b.ncol = nc;
   		for (int i = 0; i < nr; i++){
   			for (int j = 0; j < nc; j++){
   				barray[i][j] = this.matriz[i][j];
   			}
   		}
   		for (int i = 0; i < nr; i++)
   			b.index[i] = this.index[i];
   		return (Object) b;
   	}
  }

  // ADDITION
  // Add this matrix to matrix B.  This matrix remains unaltered [instance method]
  public Matriz plus(Matriz bmat){
   	if ((this.nrow != bmat.nrow)||(this.ncol != bmat.ncol)){
   		throw new IllegalArgumentException("Array dimensions do not agree");
   	}
   	int nr = bmat.nrow;
   	int nc = bmat.ncol;
   	Matriz cmat = new Matriz(nr,nc);
   	double[][] carray = cmat.getArrayReference();
   	for (int i = 0; i < nr; i++){
   		for (int j = 0; j < nc; j++){
   			carray[i][j] = this.matriz[i][j] + bmat.matriz[i][j];
   		}
   	}
   	return cmat;
  }

  // Add matrices A and B [static method]
  public static Matriz plus(Matriz amat, Matriz bmat){
   	if ((amat.nrow != bmat.nrow)||(amat.ncol != bmat.ncol)){
   		throw new IllegalArgumentException("Array dimensions do not agree");
   	}
   	int nr = amat.nrow;
   	int nc = amat.ncol;
   	Matriz cmat = new Matriz(nr,nc);
   	double[][] carray = cmat.getArrayReference();
   	for (int i = 0; i < nr; i++){
   		for (int j = 0; j < nc; j++){
   			carray[i][j] = amat.matriz[i][j] + bmat.matriz[i][j];
   		}
   	}
   	return cmat;
  }

  // Add matrix B to this matrix [equivalence of +=]
  public void plusEquals(Matriz bmat){
   	if ((this.nrow != bmat.nrow)||(this.ncol != bmat.ncol)){
   		throw new IllegalArgumentException("Array dimensions do not agree");
   	}
   	int nr = bmat.nrow;
   	int nc = bmat.ncol;
    	for (int i = 0; i < nr; i++){
   			for (int j = 0; j < nc; j++){
   				this.matriz[i][j] += bmat.matriz[i][j];
   			}
   	}
	}

  // SUBTRACTION
  // Subtract matrix B from this matrix.   
  // This matrix remains unaltered [instance method]
  public Matriz minus(Matriz bmat){
   	if ((this.nrow != bmat.nrow)||(this.ncol != bmat.ncol)){
   		throw new IllegalArgumentException("Array dimensions do not agree");
   	}
   	int nr = this.nrow;
   	int nc = this.ncol;
   	Matriz cmat = new Matriz(nr,nc);
   	double[][] carray = cmat.getArrayReference();
   	for (int i = 0; i < nr; i++){
   		for (int j = 0; j < nc; j++){
   			carray[i][j] = this.matriz[i][j] - bmat.matriz[i][j];
   		}
   	}
   	return cmat;
  }

  // Subtract matrix B from matrix A [static method]
  public static Matriz minus(Matriz amat, Matriz bmat){
   	if ((amat.nrow != bmat.nrow)||(amat.ncol != bmat.ncol)){
   		throw new IllegalArgumentException("Array dimensions do not agree");
   	}
   	int nr = amat.nrow;
   	int nc = amat.ncol;
   	Matriz cmat = new Matriz(nr,nc);
   	double[][] carray = cmat.getArrayReference();
   	for (int i = 0; i < nr; i++){
   		for (int j = 0; j < nc; j++){
   			carray[i][j] = amat.matriz[i][j] - bmat.matriz[i][j];
   		}
   	}
   	return cmat;
  }

  // Subtract matrix B from this matrix [equivlance of -=]
  public void minusEquals(Matriz bmat){
   	if ((this.nrow != bmat.nrow)||(this.ncol != bmat.ncol)){
   		throw new IllegalArgumentException("Array dimensions do not agree");
   	}
   	int nr = bmat.nrow;
   	int nc = bmat.ncol;
    	for (int i = 0; i < nr; i++){
   		for (int j = 0; j < nc; j++){
   			this.matriz[i][j] -= bmat.matriz[i][j];
   		}
   	}
  }

  // MULTIPLICATION
  // Multiply this  matrix by a matrix.   [instance method]
  // This matrix remains unaltered.
  public Matriz times(Matriz bmat){
  	if (this.ncol != bmat.nrow)
   		throw new IllegalArgumentException("Nonconformable matrices");
   	Matriz cmat = new Matriz(this.nrow, bmat.ncol);
   	double[][] carray = cmat.getArrayReference();
   	double sum = 0.0D;

   	for (int i = 0; i < this.nrow; i++){
   		for (int j = 0; j < bmat.ncol; j++){
   			sum = 0.0D;
   			for (int k = 0; k < this.ncol; k++){
   				sum += this.matriz[i][k]*bmat.matriz[k][j];
   			}
   			carray[i][j] = sum;
   		}
   	}
   	return cmat;
  }

  // Multiply this matrix by a constant [instance method]
  // This matrix remains unaltered
  public Matriz times(double constant){
   	Matriz cmat = new Matriz(this.nrow, this.ncol);
   	double [][] carray = cmat.getArrayReference();

		for (int i = 0; i < this.nrow; i++){
    	for (int j = 0; j < this.ncol; j++){
    		carray[i][j] = this.matriz[i][j]*constant;
      }
    }
   	return cmat;
  }

  // Multiply two complex matrices {static method]
  public static Matriz times(Matriz amat, Matriz bmat){
   	if (amat.ncol != bmat.nrow)
   		throw new IllegalArgumentException("Nonconformable matrices");

		Matriz cmat = new Matriz(amat.nrow, bmat.ncol);
   	double [][] carray = cmat.getArrayReference();
   	double sum = 0.0D;

   	for (int i = 0; i < amat.nrow; i++){
   		for (int j = 0; j < bmat.ncol; j++){
   			sum = 0.0D;
   			for (int k = 0; k < amat.ncol; k++){
   				sum += (amat.matriz[i][k]*bmat.matriz[k][j]);
   			}
   			carray[i][j] = sum;
   		}
   	}
   	return cmat;
  }

  // Multiply a matrix by a constant [static method]
  public static Matriz times(Matriz amat, double constant){
   	Matriz cmat = new Matriz(amat.nrow, amat.ncol);
   	double [][] carray = cmat.getArrayReference();

    for (int i = 0; i < amat.nrow; i++){
   		for (int j = 0; j < amat.ncol; j++){
   			carray[i][j] = amat.matriz[i][j]*constant;
   		}
   	}
   	return cmat;
  }

  // Multiply this matrix by a matrix [equivalence of *=]
  public void timesEquals(Matriz bmat){
   	if (this.ncol != bmat.nrow)
   		throw new IllegalArgumentException("Nonconformable matrices");

   	double sum = 0.0D;
   	for (int i = 0; i < this.nrow; i++){
   		for (int j = 0; j < bmat.ncol; j++){
   			sum = 0.0D;
   			for (int k = 0; k < this.ncol; k++){
   				sum += (this.matriz[i][k]*bmat.matriz[k][j]);
   			}
   			this.matriz[i][j] = sum;
   		}
   	}
  }

  // Multiply this matrix by a constant [equivalence of *=]
  public void timesEquals(double constant){

   	for (int i = 0; i < this.nrow; i++){
   		for (int j = 0; j < this.ncol; j++){
   			this.matriz[i][j] *= constant;
   		}
   	}
  }

  // TRANSPOSE
  // Transpose of a complex matrix [instance method]
  public Matriz transpose(){
  	Matriz tmat = new Matriz(this.ncol, this.nrow);
   	double[][] tarray = tmat.getArrayReference();
   	for (int i = 0; i < this.ncol; i++){
   		for (int j = 0; j < this.nrow; j++){
    			tarray[i][j] = this.matriz[j][i];
   		}
   	}
   	return tmat;
  }

  // Transpose of a matrix [static method]
  public static Matriz transpose(Matriz amat){
   	Matriz tmat = new Matriz(amat.ncol, amat.nrow);
   	double[][] tarray = tmat.getArrayReference();
   	for (int i = 0; i < amat.ncol; i++){
   		for (int j = 0; j < amat.nrow; j++){
   			tarray[i][j] = amat.matriz[j][i];
   		}
  	}
   	return tmat;
  }

  // OPPOSITE
  // Opposite of a matrix [instance method]
  public Matriz opposite(){
   	Matriz opp = Matriz.copy(this);
   	for (int i = 0; i < this.nrow; i++){
   		for (int j = 0; j < this.ncol; j++){
   			opp.matriz[i][j] =- this.matriz[i][j];
   		}
   	}
   	return opp;
	}
  
  // Opposite of a matrix [static method]
  public static Matriz opposite(Matriz amat){
   	Matriz opp = Matriz.copy(amat);
   	for (int i = 0; i < amat.nrow; i++){
   		for (int j = 0; j < amat.ncol; j++){
   			opp.matriz[i][j] =- amat.matriz[i][j];
   		}
   	}
   	return opp;
  }

  // TRACE
  // Trace of a  matrix [instance method]
  public double trace(){
   	double trac = 0.0D;
   	for (int i = 0; i < Math.min(this.ncol,this.ncol); i++){
   		trac += this.matriz[i][i];
  	}
   	return trac;
  }

  // Trace of a matrix [static method]
  public static double trace(Matriz amat){
   	double trac = 0.0D;
   	for (int i = 0; i < Math.min(amat.ncol,amat.ncol); i++){
   		trac += amat.matriz[i][i];
   	}
   	return trac;
  }

//
// ******************************************************************
//

	// MTODOS NECESSRIOS PARA O ALGORITMO Perona-Malik
	// Implementao: Joo Camargo Neto & Jos Iguelmar Miranda.

	// Construct matrix with a reference to an existing nrow x ncol 2-D 
	// array of int variables	(Joo 7/8/2006).
  public Matriz(int[][] twoI){
		// nmero de colunas da matriz = num linhas twoI (w)
		this.nrow = twoI.length; 
		// (h)
		this.ncol = twoI[0].length;
		this.matriz = new double[nrow][ncol];
		for (int i = 0; i < ncol; i++){
			for (int j = 0; j < nrow; j++){
				this.matriz[j][i] = (double) twoI[j][i];
			}
		}
		this.index = new int[nrow];
		for (int i = 0; i < nrow; i++)
			this.index[i] = i;
	}

	// Divide cada elemento da matriz por uma constante.
  public void divide(double constant){

   	for (int i = 0; i < this.nrow; i++){
   		for (int j = 0; j < this.ncol; j++){
        if (constant != 0.0)
   			  this.matriz[i][j] /= constant;
        else {
          String st = "Matriz::divide(double constant) -> divide by zero.";
          System.out.println(st);
        } 
   		}
   	}
  }

	// Multiplica cada um dos elementos de duas matrizes (imagens).
	// Altera os valores de 'this.'
  public void multiplica(Matriz bMat){

   	for (int i = 0; i < this.nrow; i++){
   		for (int j = 0; j < this.ncol; j++){
   			this.matriz[i][j] *= bMat.matriz[i][j];
   		}
   	}
  }

  // Eleva ao quadrado cada elemento da matriz.
  public void elevaQuadrado(){
   	for (int i = 0; i < this.nrow; i++){
   		for (int j = 0; j < this.ncol; j++){
   			this.matriz[i][j] *= this.matriz[i][j];
   		}
   	}
  }

  // Retorna o valor absoluto de cada elemento da matriz.
  public void abs(){
   	for (int i = 0; i < this.nrow; i++){
   		for (int j = 0; j < this.ncol; j++){
   			this.matriz[i][j] = Math.abs(this.matriz[i][j]);
   		}
   	}
  }

  // Retorna o exponencial de cada elemento da matriz.
  public void exp(){
   	for (int i = 0; i < this.nrow; i++){
   		for (int j = 0; j < this.ncol; j++){
   			this.matriz[i][j] = Math.exp(this.matriz[i][j]);
   		}
   	}
  }

  // Retorna o inverso de cada elemento da matriz.
  public void inverteElemento(){
   	for (int i = 0; i < this.nrow; i++){
   		for (int j = 0; j < this.ncol; j++){
   			this.matriz[i][j] = 1/this.matriz[i][j];
   		}
   	}
  }

  // Adiciona um valor a cada elemento da matriz.
  public void adiciona(double valor){
   	for (int i = 0; i < this.nrow; i++){
   		for (int j = 0; j < this.ncol; j++){
   			this.matriz[i][j] = this.matriz[i][j] + valor;
   		}
   	}
  }

	// Calcula gradiente da imagem
	// Parmetro: (Joo 7/8/2006)
	// dir = 1 ==> direo norte
	// dir = 2 ==> direo sul
	// dir = 3 ==> direo leste
	// dir = 4 ==> direo oeste
	public void gradiente(int dir) {
		double 	pReA, pReG; 
		int lin, col;

		lin = this.nrow;
		col = this.ncol;
		switch(dir){
			case 1: // direo norte para matriz e oeste para imagem
				for(int i = 0; i < col; i++){
					for(int j = lin - 1; j > 0; j--){ 
						pReA = this.matriz[j][i];
						pReG = this.matriz[j-1][i];					
						this.matriz[j][i] = pReG - pReA;
					}              
				}
				for(int i = 0; i < col; i++){
					this.matriz[0][i] = 0.0;
				}
				break;
			case 2: // direo sul para matriz e leste para imagem
				for(int i = 0; i < col; i++){
					for(int j = 0; j < lin - 1; j++){ 	
						pReA = this.matriz[j][i];
						pReG = this.matriz[j+1][i];					
						this.matriz[j][i] = pReG - pReA;
					}              
				}
				for(int i = 0; i < col; i++){
					this.matriz[lin-1][i] = 0.0;
				}
				break;
			case 3: // direo leste para matriz e norte para imagen
				for(int i = 0; i < col - 1; i++){
					for(int j = 0; j < lin; j++){ 	
						pReA = this.matriz[j][i];
						pReG = this.matriz[j][i+1];					
						this.matriz[j][i] = pReG - pReA;
					}              
				}
				for(int j = 0; j < lin; j++){
					this.matriz[j][col-1] = 0.0;
				}
				break;
			case 4:  // direo oeste para matriz e sul para imagem
				for(int i = col - 1; i > 0; i--){
					for(int j = 0; j < lin; j++){ 	
						pReA = this.matriz[j][i];
						pReG = this.matriz[j][i-1];					
						this.matriz[j][i] = pReG - pReA;
					}              
				}
				for(int j = 0; j < lin; j++){
					this.matriz[j][0] = 0.0;
				}
				break;
		}
	}

  // Imprime a matriz com 'decimais' casas decimais
  public void imprimeMatriz(int decimais) {

		String padrao = "#.";
		for (int i = 0; i < decimais; i++)
			padrao += "#";
		DecimalFormat formato = null;
		formato = new DecimalFormat(padrao);
		String elemento;

		System.out.println();
		if (this.matriz == null){
   		System.out.println("Matriz vazia.");
   	} else {
   		int nr = this.getNrow();
   		int nc = this.getNcol();
   		for (int i = 0; i < nr; i++){
   			for (int j = 0; j < nc; j++){
   				elemento = formato.format(this.matriz[i][j]);
   				System.out.print(elemento+ "\t");
   			}
				System.out.println();
   		}
   	}
  }

}
